#######################################################
#
#	Dump of installed site application's information
#
#######################################################

package SiteApp;

use strict;
use PleskVersion;

my $wrapDbh;

# Compile-time fields checking commented for work with Perl 5.0
#use fields qw(id
#			  error
#			  SiteApp
#			  Parameters
#			  Resources
#			  Package
#			  htdocs
#			  cgi_bin);

##################################################
# Parameters:
# $site_application_id, $wrapDbh_handler
##################################################
sub new( $ $ )
{
	my $class = shift;

#	my SiteApp $self = fields::new(ref $class || $class);
	my $self = {};
	bless($self, $class);

	$self->{id} = shift;
	$wrapDbh    = shift;

	$self->{SiteApp}    = {};
	$self->{Parameters} = {};
	$self->{Resources}  = {};
	$self->{Package}    = {};
	$self->{htdocs}  = [];
	$self->{cgi_bin} = [];

	my $sql = "SELECT * FROM SiteApps WHERE id='" . $self->{id} . "'";
	if (!$wrapDbh->{'EXECUTE'}->($sql)) {
		$self->_sqlError("Can not find Site Application (ID=" . $self->{id} . ")");
		return $self;
	}

	$self->{SiteApp} = $wrapDbh->{'FETCHHASH'}->();
	$wrapDbh->{'FINISH'}->();

	$self->_getParams();
	$self->_getLinkedResources();
	$self->_getPackageSpec();
	$self->_getHtdocsFiles();
	$self->_getCgibinFiles();

	return $self;
}

sub getName
{
	my SiteApp $self = shift;
	return $self->{Package}{'name'};
}

sub getVersion
{
	my SiteApp $self = shift;
	return $self->{Package}{'version'};
}

sub getRelease
{
	my SiteApp $self = shift;
	return $self->{Package}{'release'};
}

sub getDescription
{
	my SiteApp $self = shift;
	return $self->{Package}{'description'};
}

sub isCommercial
{
	my SiteApp $self = shift;
	return ($self->{Package}{'access_level'} & 1) != 0;
}

sub isIntegrated
{
	my SiteApp $self = shift;
	return 'true' eq $self->{Package}{'integrated'};
}

# Returns %hash
sub getParams
{
	my SiteApp $self = shift;
	return $self->{Parameters};
}

# Returns @list
sub getDatabases
{
	my SiteApp $self = shift;
	if (exists($self->{Resources}{'database'})) {
		return @{$self->{Resources}{'database'}};
	}
	return;
}

# Returns @list
sub getCustomButtons
{
	my SiteApp $self = shift;
	if (exists($self->{Resources}{'custom_button'})) {
		return @{$self->{Resources}{'custom_button'}};
	}
	return;
}

sub getInstallPrefix
{
	my SiteApp $self = shift;
	return $self->{SiteApp}{'install_prefix'};
}

# Returns true if site app is installed on domain
sub isDomainSiteapp
{
	my SiteApp $self = shift;
	return 'domain' eq $self->{SiteApp}{'dom_type'};
}

# Returns true if site app is installed on the given subdomain
sub isSubdomainSiteapp( $ )
{
	my SiteApp $self = shift;
	my ($subdom_id) = @_;
	return ('subdomain' eq $self->{SiteApp}{'dom_type'}) && ($subdom_id == $self->{SiteApp}->{'dom_id'});
}

# Returns true if site application is installed into ssl directory
sub isSsl
{
	my SiteApp $self = shift;
	return 'httpsdocs' eq $self->{SiteApp}{'htdocs_directory'};
}

# Returns list of files from htdocs directory, belonging
# to site application. Uses paths relative to htdocs.
# Returns @list
sub getHtdocsFiles
{
	my SiteApp $self = shift;
	return map {$self->{SiteApp}{'install_prefix'} . "/$_"} @{$self->{htdocs}};
}

# Returns list of files from cgi-bin directory, belonging
# to site application. Uses paths relative to cgi-bin.
# Returns @list
sub getCgibinFiles
{
	my SiteApp $self = shift;
	return map {$self->{SiteApp}{'install_prefix'} . "/$_"} @{$self->{cgi_bin}};
}

# Returns error message, indicating erroneous state of the object
sub error
{
	my SiteApp $self = shift;
	return $self->{error};
}

sub _getHtdocsFiles
{
	my SiteApp $self = shift;
	# Prior to 7.5.4 there were no SiteAppFiles table
	if (!PleskVersion::atLeast(7, 5, 4)) {
		$self->{htdocs} = ['.'];
		return $self->{htdocs};
	}

	my $sql = "SELECT file FROM SiteAppFiles WHERE instance_id='" . $self->{id} . "' AND prefix='htdocs'";
	if ($wrapDbh->{'EXECUTE'}->($sql)) {
		while (my $ptrRow = $wrapDbh->{'FETCHROW'}->()) {
			push @{$self->{htdocs}}, $ptrRow->[0];
		}
	}
	$wrapDbh->{'FINISH'}->();

	return $self->{htdocs};
}

sub _getCgibinFiles
{
	my SiteApp $self = shift;
	# Prior to 7.5.4 there were no SiteAppFiles table
	if (!PleskVersion::atLeast(7, 5, 4)) {
		$self->{cgi_bin} = ['.'];
		return $self->{cgi_bin};
	}

	my $sql = "SELECT file FROM SiteAppFiles WHERE instance_id='" . $self->{id} . "' AND prefix='cgi-bin'";
	if ($wrapDbh->{'EXECUTE'}->($sql)) {
		while (my $ptrRow = $wrapDbh->{'FETCHROW'}->()) {
			push @{$self->{cgi_bin}}, $ptrRow->[0];
		}
	}
	$wrapDbh->{'FINISH'}->();

	return $self->{cgi_bin};
}

sub _sqlError( $ )
{
	my SiteApp $self = shift;
	my ($message) = @_;
	$wrapDbh->{'FINISH'}->();
	$self->{error} = $message;
}

sub _getParams
{
	my SiteApp $self = shift;
	my $sql = "SELECT parameter, value FROM Parameters WHERE id='" . ${$self->{SiteApp}}{'params_id'} . "'";
	if ($wrapDbh->{'EXECUTE'}->($sql)) {
		while (my $ptrRow = $wrapDbh->{'FETCHROW'}->()) {
			$self->{Parameters}{$ptrRow->[0]} = $ptrRow->[1];
		}
		$wrapDbh->{'FINISH'}->();
	}
}

sub _getLinkedResources
{
	my SiteApp $self = shift;
	my $sql = "SELECT type, res_id FROM SiteAppResources WHERE app_id='" . $self->{id} . "'";
	if ($wrapDbh->{'EXECUTE'}->($sql)) {
		while (my $ptrRow = $wrapDbh->{'FETCHROW'}->()) {
			push @{$self->{Resources}{$ptrRow->[0]}}, $ptrRow->[1];
		}
		$wrapDbh->{'FINISH'}->();
	}
}

sub _getPackageSpec
{
	my SiteApp $self = shift;
	my $sql = "SELECT * FROM SiteAppPackages WHERE id='" . $self->{SiteApp}{'app_id'} . "'";
	if (!$wrapDbh->{'EXECUTE'}->($sql)) {
		return $self->_sqlError("Can not find package for Site Application (ID=" . $self->{id} . ")");
	}
	$self->{Package} = $wrapDbh->{'FETCHHASH'}->();
	$wrapDbh->{'FINISH'}->();
}

1;
