import sys, unpacker, split_files, string

try:
    import subprocess
except ImportError:
    import subprocess_compat
    subprocess = subprocess_compat

def usage():
    print "Usage: cid-unpacker <session-dir>. Stdin is command."

def main():
    if len(sys.argv) != 2:
        usage()
        sys.exit(1)

    sessionDir = sys.argv[1].decode("UTF-8")

    cmd = string.split(sys.stdin.read(), "\0")
    (command, args) = cmd[0], cmd[1:]

    if command == "untar":
        if len(args) != 2:
            sys.stderr.write("'untar' command requires 2 arguments\n")
            sys.exit(1)

        id_str = args[0].decode("UTF-8")

        #
        # FreeBSD up to 5.3 has bug: read(2) on blocking pipe
        # sometimes return EAGAIN.  GNU tar is not ready to handle
        # this, so we just restart tar three times (arbitrary value, but
        # sounds reasonable) if error occured
        #
        # This workaround should be removed as FreeBSD 4.11 is dropped
        #

        for lastAttempt in (False, False, False, True):
            try:
                id = unpacker.Id(id_str)
                tarParts = split_files.findParts(sessionDir + "/archives/" + id.filename)
                fd = split_files.open(tarParts)

                files = unpacker.TarUnpacker(args[1]).unpack(fd.stdout, id_str)
                sys.stdout.write("\0".join(files))
            except Exception, e:
                sys.stderr.write("Runtime error during 'untar': " + unicode(e) + "\n")
                if lastAttempt:
                    sys.exit(1)
                else:
                    sys.stderr.write("One more attempt to untar\n")
            else:
                sys.exit(0)

    if command == "db":
        if len(args) != 7:
            sys.stderr.write("'db' command requires 7 arguments\n")
            sys.exit(1)

        id_str = args[0].decode("UTF-8")

        try:
            id = unpacker.Id(id_str)
            dbParts = split_files.findParts(sessionDir + "/archives/" + id.filename)
            fd = split_files.open(dbParts)

            up = unpacker.DbUnpacker(args[1], args[2], args[3], args[4], args[5], args[6])
            up.unpack(fd.stdout, id_str)
        except Exception, e:
            sys.stderr.write("Runtime error during 'db': " + unicode(e) + "\n")
            sys.exit(1)
        sys.exit(0)

    if command == "unpack":
        if len(args) != 2:
            sys.stderr.write("'unpack' command requires 2 arguments\n")
            sys.exit(1)

        id_str = args[0].decode("UTF-8")

        try:
            id = unpacker.Id(id_str)
            tarParts = split_files.findParts(sessionDir + "/archives/" + id.filename)
            fd = split_files.open(tarParts)

            up = unpacker.PartUnpacker(args[1]).unpack(fd.stdout, id_str)
        except Exception, e:
            sys.stderr.write("Runtime error during 'unpack': " + unicode(e) + "\n")
            sys.exit(1)
        sys.exit(0)

    sys.stderr.write("Unknown command: " + command + "\n")
    sys.exit(1)


if __name__ == '__main__':
    main()
